export interface User {
  id: string;
  name: string;
  currentScore: number;
  highScore: number;
  highScoreDate: Date;
}

const db = new Map<string, User>();

// Clear user with 0 highscore every 15 minutes to avoid filling the memory.
// This is not part of the challenge, I'm just too lazy to deploy a database
setInterval(() => {
  db.forEach((user) => {
    if (user.highScore === 0) {
      db.delete(user.id);
    }
  });
}, 1000 * 60 * 60 * 15);

export default {
  createUser(name: string) {
    const user: User = {
      id: crypto.randomUUID(),
      name: String(name).trim().slice(0, 40) || "Anonymous",
      currentScore: 0,
      highScore: 0,
      highScoreDate: new Date(),
    };
    db.set(user.id, user);
    return user.id;
  },

  getUser(id: string) {
    return db.get(id);
  },

  getScoreboard() {
    return Array.from(db.values())
      .sort((a, b) => b.highScore - a.highScore)
      .map((user, idx) => ({
        rank: idx + 1,
        name: user.name,
        highScore: user.highScore,
        highScoreDate: user.highScoreDate,
      }));
  },
};
