import express from "express";
import cookieParser from "cookie-parser";
import db from "./db";
import quizz from "./quizz";
import middlewares from "./middlewares";

const port = 8000;
const app = express();
app.use(cookieParser());
app.use(express.json());
app.use(middlewares.superJson);

app.get("/api/scoreboard", async (req, res) => {
  return res.json(db.getScoreboard());
});

app.post("/api/new", async (req, res) => {
  return res.cookie("user", db.createUser(req.body.name)).redirect("/play.html");
});

app.get("/api/me", middlewares.assertSession, async (req, res) => {
  return res.json(req.user);
});

app.post("/api/play", middlewares.assertSession, async (req, res) => {
  try {
    const question = req.body.question;
    const answer = req.body.answer;

    const correct = quizz.checkAnswer(question, answer);

    if (correct) {
      req.user.currentScore++;
      if (req.user.currentScore > req.user.highScore) {
        req.user.highScore = req.user.currentScore;
        req.user.highScoreDate = new Date();
      }
    } else {
      req.user.currentScore = 0;
    }

    const response = {
      correct,
      yourAnswer: answer,
      user: req.user,
    };

    return res.json(response);
  } catch (e) {
    console.error(e);
    return res.status(400).send("Bad request"); // it's always the user's fault
  }
});

app.use(express.static("public"));
app.listen(port, () => {
  console.log(`Super Rock Quizz started at http://localhost:${port}`);
});
