import type { ErrorRequestHandler, NextFunction, Request, Response } from "express";
import db, { type User } from "./db";
import SuperJSON from "superjson";

declare global {
  namespace Express {
    export interface Request {
      user: User;
    }
  }
}

export default {
  assertSession(req: Request, res: Response, next: NextFunction) {
    // Check if the user cookie is set and if the user exists in the database.
    const user = db.getUser(req.cookies.user);
    if (!user) {
      return res.redirect("/");
    }

    req.user = user;
    next();
  },
  superJson(req: Request, res: Response, next: NextFunction) {
    // User superjson since we are a super quizz.
    if (req.body) {
      try {
        req.body = SuperJSON.deserialize(req.body);
      } catch (e) {
        return res.status(400).send("You are not super enough to play this quizz.");
      }
    }
    next();
  },
};
